align		macro n
			org ($ + n - 1) / n * n
			endm
			
			org 0x8200
			
			; fill screen with texture
			ld hl,0x4000
			ld de,0x4001
			ld bc,0x0100
			ld (hl),0xaa
			ldir
			ld (hl),l
			ld bc,0x00ff
			ldir
			
			ld hl,0x4000
			ld bc,0x0600
			ldir
			ld hl,0x4000
			ld de,0x5000
			ld bc,0x0800
			ldir
			ld hl,0x5900
			ld de,0x5901
			ld (hl),7
			ld bc,0x00ff
			ldir
			
			ld hl,0x4800
			ld de,0x4801
			ld (hl),l
			ld bc,0x07ff
			ldir
			
;squirt our interrupt routine into place
interrupt_init
			include "interrupt_81.sym"
			ld a,0xcd		; CALL
			ld (custom_int_81),a
			ld hl,interrupt
			ld (custom_int_81 + 1),hl
			
discolp
			halt
			
			ld a,(xoff+1)
			ld (xoff_local+1),a
			ld a,(yoff+1)
			ld (yoff_local+1),a
			
			ld hl,(sine_pos+1)
			push hl
			ld e,(hl)
			ld bc,ksintheta
			call mkmultable		; build a multiple-of-sines table
			pop hl

			; now do the same for cosine
			ld a,l
			add a,64
			ld l,a
			ld e,(hl)
			inc b
			call mkmultable
			
			ld a,texture2/0x0100
			ld (texptr+1),a
			ld bc,0x5800
			ld hl,perspect
			call project
			ld a,texture/0x0100
			ld (texptr+1),a
			ld bc,0x5a00
			ld hl,perspect+0x0200
			call project
trapdoor
			jr discolp
			ret
			
			
project
project_lp
			push hl
			push bc
			ld e,(hl)	; get floor X coordinate
			inc h
			ld d,(hl)	; get floor Y coordinate
			ld h,ksintheta/0x0100
			ld l,d
			ld a,(hl)	; get y*sin(t)
			inc h
			ld l,e		; get x*cos(t)
			add a,(hl)
xoff_local	add a,0		; adjust for x0
			srl a		; keep top 4 bits for texture
			srl a
			srl a
			; srl a
			and 0x0f	; actually keep bits 3-6
			ld b,a

			ld l,d		; get y*cos(t)
			ld a,(hl)
			dec h
			ld l,e		; get x*sin(t)
			sub (hl)
yoff_local	add a,0		; adjust for y0
			and 0xf0
			or b			
			; look up in texture map
texptr
			ld h,texture/0x0100
			ld l,a
			ld a,(hl)

			pop bc

			ld (bc),a
			pop hl
			inc l
			inc c
			jp nz,project_lp
			ret

interrupt
			push af
			push bc
			push de
			push hl
xoff		ld a,0
			inc a
			ld (xoff+1),a
yoff		ld a,0
			add a,4
			ld (yoff+1),a
sine_pos	ld hl,bigsine
			inc l
			ld (sine_pos+1),hl
			call scroller
			call timebomb
			
			pop hl
			pop de
			pop bc
			pop af
			ret
			
timebomb	ld hl,610*2 ; 640*2
			dec hl
			ld (timebomb+1),hl
			ld a,h
			or l
			ret nz
			ld hl,0x0000
			ld (trapdoor),hl
			ret

; make a table consisting of multiples of the signed value in e	
mkmultable
			; first sign-extend de
			sla e
			sbc a,a
			ld d,a
			; now fill the table
			ld hl,0
mkmultable_lp
			ld a,h
			ld (bc),a
			add hl,de
			inc c
			jp nz,mkmultable_lp
			ret

no_scroll_yet
			ld (scroll_countdown+1),hl
			ret

scroller
			; don't call for the first 640 frames. or 610, or something
scroll_countdown
			ld hl,610
			dec hl
			ld a,h
			or l
			jr nz,no_scroll_yet
scrollpix	ld a,1
			dec a
			and 1
			ld (scrollpix+1),a
			jp nz,skip_scroll_char
scrollpos	ld hl,scrolltext
			ld a,(hl)
			or a
			jr z,scroll_wrap
			inc hl
			ld (scrollpos+1),hl
			ld l,a
			ld h,0
			add hl,hl
			add hl,hl
			add hl,hl
			ld de,arcade_font-0x0100
			add hl,de
			ld de,scrollbuf
			rept 8
				ldi
			endm
scroll_wrap

scrollpos2	ld hl,scrolltext2
			ld a,(hl)
			or a
			jr z,scroll_wrap2
			inc hl
			ld (scrollpos2+1),hl
			ld l,a
			ld h,0
			add hl,hl
			add hl,hl
			add hl,hl
			ld de,arcade_font-0x0100
			add hl,de
			ld de,scrollbuf+8
			rept 8
				ldi
			endm
scroll_wrap2

skip_scroll_char
			; call scroll1
scroll1
			ld hl,0x485f
			ld de,scrollbuf
			call scroll_line
			ld hl,0x489f
scroll_line
			ld b,8
scroller_lp
			push hl

			ex de,hl
			xor a
			rld
			ex de,hl
			
			rept 31
				rld
				dec l
			endm
			rld
			
			pop hl
			inc h
			inc de
			djnz scroller_lp
			ret
			
scrollbuf	ds 16

			align 0x0100
bigsine
			include "bigsine.asm"
perspect
			include "perspect.asm"
ksintheta
			ds 0x0100
kcostheta
			ds 0x0100
			
texture
			incbin "texture.bin"
texture2
			incbin "texture2.bin"
			
scrolltext	defb "RC55   UKSCENE   RAWW ARSE   BDSE   TULOU   DEVIANCE   TZT   BLUESPOON   XPLSV   LEBLANE   AY RIDERS   CPU   ZILOG   4D   TRIEBKRAFT   CI5   UVS   MB-MANIAX   TCG   BRAINWAVE   CONSPIRACY   RIBBON   TBL   POUET.NET "
			defb 0
scrolltext2	defb "   SUNDOWN   ATE BIT   TRIAD   FAIRLIGHT   DCS   ASD   APPROXIMATE   COSINE   MADWIZARDS   TYRANID   JUSTINAS   WAZ   H-PRG   ZEROTEAM   3SC   EXIN   MISTER BEEP   SKRJU   SERZH   MAALI   FARBRAUSCH   SCENE.ORG   UNTERGRUND.NET "
			defb 0
			
arcade_font
			incbin "arcade_font.bin"